All of lore.kernel.org
 help / color / mirror / Atom feed
* Patch "net: fix socket refcounting in skb_complete_wifi_ack()" has been added to the 4.4-stable tree
@ 2017-03-18 14:45 gregkh
  0 siblings, 0 replies; only message in thread
From: gregkh @ 2017-03-18 14:45 UTC (permalink / raw)
  To: edumazet, alexander.h.duyck, davem, gregkh, johannes, soheil, willemb
  Cc: stable, stable-commits


This is a note to let you know that I've just added the patch titled

    net: fix socket refcounting in skb_complete_wifi_ack()

to the 4.4-stable tree which can be found at:
    http://www.kernel.org/git/?p=linux/kernel/git/stable/stable-queue.git;a=summary

The filename of the patch is:
     net-fix-socket-refcounting-in-skb_complete_wifi_ack.patch
and it can be found in the queue-4.4 subdirectory.

If you, or anyone else, feels it should not be added to the stable tree,
please let <stable@vger.kernel.org> know about it.


>From foo@baz Sat Mar 18 22:38:09 CST 2017
From: Eric Dumazet <edumazet@google.com>
Date: Fri, 3 Mar 2017 21:01:02 -0800
Subject: net: fix socket refcounting in skb_complete_wifi_ack()

From: Eric Dumazet <edumazet@google.com>


[ Upstream commit dd4f10722aeb10f4f582948839f066bebe44e5fb ]

TX skbs do not necessarily hold a reference on skb->sk->sk_refcnt
By the time TX completion happens, sk_refcnt might be already 0.

sock_hold()/sock_put() would then corrupt critical state, like
sk_wmem_alloc.

Fixes: bf7fa551e0ce ("mac80211: Resolve sk_refcnt/sk_wmem_alloc issue in wifi ack path")
Signed-off-by: Eric Dumazet <edumazet@google.com>
Cc: Alexander Duyck <alexander.h.duyck@intel.com>
Cc: Johannes Berg <johannes@sipsolutions.net>
Cc: Soheil Hassas Yeganeh <soheil@google.com>
Cc: Willem de Bruijn <willemb@google.com>
Acked-by: Soheil Hassas Yeganeh <soheil@google.com>
Signed-off-by: David S. Miller <davem@davemloft.net>
Signed-off-by: Greg Kroah-Hartman <gregkh@linuxfoundation.org>
---
 net/core/skbuff.c |   15 ++++++++-------
 1 file changed, 8 insertions(+), 7 deletions(-)

--- a/net/core/skbuff.c
+++ b/net/core/skbuff.c
@@ -3735,7 +3735,7 @@ void skb_complete_wifi_ack(struct sk_buf
 {
 	struct sock *sk = skb->sk;
 	struct sock_exterr_skb *serr;
-	int err;
+	int err = 1;
 
 	skb->wifi_acked_valid = 1;
 	skb->wifi_acked = acked;
@@ -3745,14 +3745,15 @@ void skb_complete_wifi_ack(struct sk_buf
 	serr->ee.ee_errno = ENOMSG;
 	serr->ee.ee_origin = SO_EE_ORIGIN_TXSTATUS;
 
-	/* take a reference to prevent skb_orphan() from freeing the socket */
-	sock_hold(sk);
-
-	err = sock_queue_err_skb(sk, skb);
+	/* Take a reference to prevent skb_orphan() from freeing the socket,
+	 * but only if the socket refcount is not zero.
+	 */
+	if (likely(atomic_inc_not_zero(&sk->sk_refcnt))) {
+		err = sock_queue_err_skb(sk, skb);
+		sock_put(sk);
+	}
 	if (err)
 		kfree_skb(skb);
-
-	sock_put(sk);
 }
 EXPORT_SYMBOL_GPL(skb_complete_wifi_ack);
 


Patches currently in stable-queue which might be from edumazet@google.com are

queue-4.4/tcp-fix-various-issues-for-sockets-morphing-to-listen-state.patch
queue-4.4/dccp-fix-use-after-free-in-dccp_feat_activate_values.patch
queue-4.4/net-fix-socket-refcounting-in-skb_complete_tx_timestamp.patch
queue-4.4/net-net_enable_timestamp-can-be-called-from-irq-contexts.patch
queue-4.4/dccp-unlock-sock-before-calling-sk_free.patch
queue-4.4/net-don-t-call-strlen-on-the-user-buffer-in-packet_bind_spkt.patch
queue-4.4/net-fix-socket-refcounting-in-skb_complete_wifi_ack.patch

^ permalink raw reply	[flat|nested] only message in thread

only message in thread, other threads:[~2017-03-18 14:46 UTC | newest]

Thread overview: (only message) (download: mbox.gz / follow: Atom feed)
-- links below jump to the message on this page --
2017-03-18 14:45 Patch "net: fix socket refcounting in skb_complete_wifi_ack()" has been added to the 4.4-stable tree gregkh

This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.